<?php
/**
 * CoinMarketCap related hooks and functions
 *
 * @since 1.0.0
 */

/**
 * Returns a list of cryptocurrency CoinMarketCap IDs, and their associated labels.
 *
 * This function queries the CoinMarketCap API.
 *
 * @since 1.0.0
 *
 * @return array
 */
function ignition_blockchain_coinmarketcap_get_cryptocurrency_choices() {

	$choices = array(
		'1' => __( 'Bitcoin (BTC)', 'ignition-blockchain' ),
	);

	$coins = ignition_blockchain_coinmarketcap_get_cryptocurrencies();

	if ( ! empty( $coins ) ) {
		$choices = array();
		$coins   = wp_list_sort( $coins, 'name' );

		foreach ( $coins as $coin ) {
			/* translators: %1$s is the coin's name, e.g. Bitcoin. %2$s is the coin's symbol, e.g. BTC. */
			$choices[ (string) $coin['id'] ] = sprintf( _x( '%1$s (%2$s)', 'cryptocoin label', 'ignition-blockchain' ),
				$coin['name'],
				$coin['symbol']
			);
		}
	}

	return $choices;
}

/**
 * Returns an array of cryptocurrencies and symbol-related data.
 *
 * This function queries the CoinMarketCap API.
 *
 * @since 1.0.0
 *
 * @return array
 */
function ignition_blockchain_coinmarketcap_get_cryptocurrencies() {

	$trans_name = 'ignition_blockchain_coinmarketcap_cryptocurrencies';
	$coins      = get_transient( $trans_name );

	if ( false === $coins ) {

		$endpoint = 'cryptocurrency/map';

		$response = ignition_blockchain_coinmarketcap_get_data( $endpoint );

		$coins = array();

		if ( ! $response['error'] && ! empty( $response['data']['data'] ) ) {
			$coins = $response['data']['data'];
		}

		if ( empty( $coins ) ) {
			// There was a problem, so don't cache anything. Fall back to the ignition_blockchain_coinmarketcap_do_api_call() transients.
			delete_transient( $trans_name );
		} else {
			set_transient( $trans_name, $coins, 24 * HOUR_IN_SECONDS );
		}
	}

	return $coins;
}

/**
 * Returns an array of fiat currencies.
 *
 * @since 1.0.0
 *
 * @return array
 */
function ignition_blockchain_coinmarketcap_get_fiat_currencies() {
	/**
	 * Filters the list of fiat currencies.
	 *
	 * @since 1.0.0
	 *
	 * @param array $currencies
	 */
	return apply_filters( 'ignition_blockchain_coinmarketcap_fiat_currencies', array(
		'AUD' => esc_html_x( 'AUD', 'currency code', 'ignition-blockchain' ),
		'BRL' => esc_html_x( 'BRL', 'currency code', 'ignition-blockchain' ),
		'CAD' => esc_html_x( 'CAD', 'currency code', 'ignition-blockchain' ),
		'CHF' => esc_html_x( 'CHF', 'currency code', 'ignition-blockchain' ),
		'CLP' => esc_html_x( 'CLP', 'currency code', 'ignition-blockchain' ),
		'CNY' => esc_html_x( 'CNY', 'currency code', 'ignition-blockchain' ),
		'CZK' => esc_html_x( 'CZK', 'currency code', 'ignition-blockchain' ),
		'DKK' => esc_html_x( 'DKK', 'currency code', 'ignition-blockchain' ),
		'EUR' => esc_html_x( 'EUR', 'currency code', 'ignition-blockchain' ),
		'GBP' => esc_html_x( 'GBP', 'currency code', 'ignition-blockchain' ),
		'HKD' => esc_html_x( 'HKD', 'currency code', 'ignition-blockchain' ),
		'HUF' => esc_html_x( 'HUF', 'currency code', 'ignition-blockchain' ),
		'IDR' => esc_html_x( 'IDR', 'currency code', 'ignition-blockchain' ),
		'ILS' => esc_html_x( 'ILS', 'currency code', 'ignition-blockchain' ),
		'INR' => esc_html_x( 'INR', 'currency code', 'ignition-blockchain' ),
		'JPY' => esc_html_x( 'JPY', 'currency code', 'ignition-blockchain' ),
		'KRW' => esc_html_x( 'KRW', 'currency code', 'ignition-blockchain' ),
		'MXN' => esc_html_x( 'MXN', 'currency code', 'ignition-blockchain' ),
		'MYR' => esc_html_x( 'MYR', 'currency code', 'ignition-blockchain' ),
		'NOK' => esc_html_x( 'NOK', 'currency code', 'ignition-blockchain' ),
		'NZD' => esc_html_x( 'NZD', 'currency code', 'ignition-blockchain' ),
		'PHP' => esc_html_x( 'PHP', 'currency code', 'ignition-blockchain' ),
		'PKR' => esc_html_x( 'PKR', 'currency code', 'ignition-blockchain' ),
		'PLN' => esc_html_x( 'PLN', 'currency code', 'ignition-blockchain' ),
		'RUB' => esc_html_x( 'RUB', 'currency code', 'ignition-blockchain' ),
		'SEK' => esc_html_x( 'SEK', 'currency code', 'ignition-blockchain' ),
		'SGD' => esc_html_x( 'SGD', 'currency code', 'ignition-blockchain' ),
		'THB' => esc_html_x( 'THB', 'currency code', 'ignition-blockchain' ),
		'TRY' => esc_html_x( 'TRY', 'currency code', 'ignition-blockchain' ),
		'TWD' => esc_html_x( 'TWD', 'currency code', 'ignition-blockchain' ),
		'USD' => esc_html_x( 'USD', 'currency code', 'ignition-blockchain' ),
		'ZAR' => esc_html_x( 'ZAR', 'currency code', 'ignition-blockchain' ),
	) );
}

/**
 * Returns a number formatted according to a currency's format.
 *
 * @since 1.0.0
 *
 * @param numeric $number   The number to format.
 * @param string  $currency Optional. Currency code whose format should be used to format $number. Default 'USD'.
 * @param int     $decimals Optional. Precision of the number of decimal places. Default 2.
 *
 * @return array
 */
function ignition_blockchain_coinmarketcap_format_currency_number( $number, $currency = 'USD', $decimals = 2 ) {
	/**
	 * Filters the array of fiat currencies's format strings.
	 *
	 * @since 1.0.0
	 *
	 * @param array $currencies
	 */
	$currencies = apply_filters( 'ignition_blockchain_coinmarketcap_currencies_format', array(
		'USD' => '$%s',
		'AUD' => '$%s',
		'BRL' => 'R$%s',
		'CAD' => '$%s',
		'CHF' => 'CHF%s',
		'CLP' => '$%s',
		'CNY' => '¥%s',
		'CZK' => 'Kč%s',
		'DKK' => 'kr.%s',
		'EUR' => '€%s',
		'GBP' => '£%s',
		'HKD' => '$%s',
		'HUF' => 'Ft%s',
		'IDR' => 'Rp%s',
		'ILS' => '₪%s',
		'INR' => '₹%s',
		'JPY' => '¥%s',
		'KRW' => '₩%s',
		'MXN' => '$%s',
		'MYR' => 'RM%s',
		'NOK' => 'kr%s',
		'NZD' => '$%s',
		'PHP' => '₱%s',
		'PKR' => '₨%s',
		'PLN' => 'zł%s',
		'RUB' => '₽%s',
		'SEK' => 'kr%s',
		'SGD' => '$%s',
		'THB' => '฿%s',
		'TRY' => '₺%s',
		'TWD' => 'NT$%s',
		'ZAR' => 'R%s',
	) );

	if ( ! array_key_exists( $currency, $currencies ) ) {
		$currency = 'USD';
	}

	$value = sprintf( $currencies[ $currency ], number_format_i18n( $number, $decimals ) );

	/**
	 * Filters the currency-formatted number.
	 *
	 * @since 1.0.0
	 *
	 * @param string  $value
	 * @param numeric $number
	 * @param string  $currency
	 * @param int     $decimals
	 * @param array   $currencies
	 */
	$value = apply_filters( 'ignition_blockchain_coinmarketcap_format_currency_number', $value, $number, $currency, $decimals, $currencies );

	return $value;
}

/**
 * Sanitizes a fiat currency code.
 *
 * @since 1.0.0
 *
 * @param string $value Input value to sanitize.
 *
 * @return string
 */
function ignition_blockchain_coinmarketcap_sanitize_fiat_currency( $value ) {
	$choices = ignition_blockchain_coinmarketcap_get_fiat_currencies();
	if ( array_key_exists( $value, $choices ) ) {
		return $value;
	}

	return ignition_blockchain_coinmarketcap_default_fiat_currency();
}

/**
 * Returns the default fiat currency.
 *
 * @since 1.0.0
 *
 * @return string
 */
function ignition_blockchain_coinmarketcap_default_fiat_currency() {
	/**
	 * Filters the default fiat currency.
	 *
	 * @since 1.0.0
	 *
	 * @param string $currency
	 */
	return apply_filters( 'ignition_blockchain_coinmarketcap_default_fiat_currency', 'USD' );
}

/**
 * Queries, wraps and returns a CoinMarketCap API request.
 *
 * @since 1.0.0
 *
 * @param string $endpoint
 * @param array  $params
 *
 * @return array
 */
function ignition_blockchain_coinmarketcap_get_data( $endpoint, $params = array() ) {

	$endpoint = trim( $endpoint );

	$params = is_array( $params ) ? $params : array();
	if ( ! empty( $params['convert'] ) && 'USD' === $params['convert'] ) {
		unset( $params['convert'] );
	}

	if ( empty( $endpoint ) ) {
		$response = array(
			'error'  => true,
			'errors' => array( 'No endpoint defined.' ),
			'data'   => new stdClass(),
		);

		return $response;
	}

	$data = ignition_blockchain_coinmarketcap_do_api_call( $endpoint, $params );

	if ( is_wp_error( $data ) ) {
		$response = array(
			'error'  => true,
			'errors' => $data->get_error_messages(),
			'data'   => new stdClass(),
		);
	} elseif ( ! empty( $data ) && isset( $data['response']['code'] ) && 200 === $data['response']['code'] ) {
		$response = array(
			'error'  => false,
			'errors' => array(),
			'data'   => json_decode( $data['body'], true ),
		);
	} else {
		$json = json_decode( $data['body'], true );
		if ( isset( $json['error'] ) ) {
			$response = array(
				'error'  => true,
				'errors' => array( $json['error'] ),
				'data'   => new stdClass(),
			);
		} elseif ( isset( $json['status']['error_message'] ) ) {
			$response = array(
				'error'  => true,
				'errors' => array( $json['status']['error_message'] ),
				'data'   => new stdClass(),
			);
		} else {
			$response = array(
				'error'  => true,
				'errors' => array( 'CoinMarketCap.com Error' ),
				'data'   => new stdClass(),
			);
		}
	}

	return $response;
}

/**
 * Queries the CoinMarketCap API and caches the responses.
 *
 * You might want to use the more high-level ignition_blockchain_coinmarketcap_get_data() instead.
 *
 * @since 1.0.0
 *
 * @param string $endpoint
 * @param array  $params
 * @param bool   $bypass_cache Optional. When true, the API will always be queried. Default false.
 *
 * @return array
 */
function ignition_blockchain_coinmarketcap_do_api_call( $endpoint, $params, $bypass_cache = false ) {
	$api_key    = get_theme_mod( 'theme_coinmarketcap_api_key', ignition_blockchain_ignition_customizer_defaults( 'theme_coinmarketcap_api_key' ) );
	$query_hash = ignition_blockchain_coinmarketcap_get_query_hash( $endpoint, $params, $api_key );
	$trans_name = ignition_blockchain_coinmarketcap_get_transient_name( $query_hash );

	/**
	 * Filters the time (in seconds) that the API response will be cached for.
	 *
	 * @since 1.0.0
	 *
	 * @param int $cache_time
	 */
	$cache_time = apply_filters( 'ignition_blockchain_coinmarketcap_query_cache_time', 30 * MINUTE_IN_SECONDS );

	/**
	 * Filters the retry time (in seconds) after an API error.
	 *
	 * @since 1.0.0
	 *
	 * @param int $retry_time
	 */
	$retry_time = apply_filters( 'ignition_blockchain_coinmarketcap_query_retry_time', 2 * MINUTE_IN_SECONDS ); // Retry after failure.

	/**
	 * Filters the request timeout (in seconds) for the API call.
	 *
	 * @since 1.0.0
	 *
	 * @param int $request_timeout
	 */
	$request_timeout = apply_filters( 'ignition_blockchain_coinmarketcap_query_timeout', 30 );

	// This transient will never expire. Holds the last known good response for fallback.
	$noexp_name = ignition_blockchain_coinmarketcap_get_nonexpiring_transient_name( $query_hash );

	$api_url = 'https://pro-api.coinmarketcap.com/v1/';

	$response = get_transient( $trans_name );
	if ( false === $response || $bypass_cache ) {

		$url = $api_url . $endpoint;

		// The API requires that the URL does NOT have a trailing forward slash.
		$url = untrailingslashit( $url );
		$url = add_query_arg( $params, $url );

		$request_params = array(
			'timeout' => $request_timeout,
		);

		if ( $api_key ) {
			$request_params['headers'] = array(
				'X-CMC_PRO_API_KEY' => $api_key,
			);
		}

		$response = wp_safe_remote_get( $url, $request_params );

		if ( ! $bypass_cache ) {
			$noexp = (array) get_transient( $noexp_name );

			if ( ! is_wp_error( $response ) && ! empty( $response ) && isset( $response['response']['code'] ) && 200 === (int) $response['response']['code'] ) {
				$json = json_decode( $response['body'], true );

				if ( ! is_null( $json ) ) {
					$noexp['last_good_response']  = $response;
					$noexp['last_good_timestamp'] = current_time( 'timestamp', true );
				} else {
					$cache_time = $retry_time;
				}
			}

			if ( is_wp_error( $response ) ) {
				$noexp['last_fail_message']   = $response->get_error_messages();
				$noexp['last_fail_timestamp'] = current_time( 'timestamp', true );

				$cache_time = $retry_time;

				if ( ! empty( $noexp['last_good_response'] ) ) {
					$response = $noexp['last_good_response'];
				}
			}

			// Cache indefinitely, both for a fallback to $trans_name, as well as debugging.
			set_transient( $noexp_name, $noexp, 0 );

			set_transient( $trans_name, $response, $cache_time );
		}
	}

	return $response;
}

/**
 * Returns a hash that uniquely identifies an API call.
 *
 * @since 1.0.0
 *
 * @param string $endpoint
 * @param array  $params
 * @param string $api_key
 *
 * @return string
 */
function ignition_blockchain_coinmarketcap_get_query_hash( $endpoint, $params, $api_key = '' ) {
	$hash = sanitize_text_field( md5( $api_key . $endpoint . wp_json_encode( $params ) ) );

	/**
	 * Filters the generated API query hash.
	 *
	 * @since 1.0.0
	 *
	 * @param string $hash
	 * @param string $endpoint
	 * @param array  $params
	 */
	return apply_filters( 'ignition_blockchain_coinmarketcap_query_hash', $hash, $endpoint, $params );
}

/**
 * Returns a transient name for a give API call hash.
 *
 * @since 1.0.0
 *
 * @param string $query_hash
 *
 * @return string
 */
function ignition_blockchain_coinmarketcap_get_transient_name( $query_hash ) {
	$base_name = 'ignition_blockchain_coinmarketcap_%s';
	$name      = sprintf( $base_name, $query_hash );

	/**
	 * Filters the generated transient name.
	 *
	 * @since 1.0.0
	 *
	 * @param string $name
	 * @param string $base_name
	 * @param string $query_hash
	 */
	return apply_filters( 'ignition_blockchain_coinmarketcap_transient_name', $name, $base_name, $query_hash );
}

/**
 * Returns a non-expiring transient name for a give API call hash.
 *
 * @since 1.0.0
 *
 * @param string $query_hash
 *
 * @return string
 */
function ignition_blockchain_coinmarketcap_get_nonexpiring_transient_name( $query_hash ) {
	$base_name = 'ignition_blockchain_coinmarketcap_noexp_%s';
	$name      = sprintf( $base_name, $query_hash );

	/**
	 * Filters the generated transient name.
	 *
	 * @since 1.0.0
	 *
	 * @param string $name
	 * @param string $base_name
	 * @param string $query_hash
	 */
	return apply_filters( 'ignition_blockchain_coinmarketcap_nonexpiring_transient_name', $name, $base_name, $query_hash );
}

/**
 * Converts a GMT timestamp to a localized timestamp.
 *
 * @since 1.0.0
 *
 * @param string $gmt_timestamp
 *
 * @return string
 */
function ignition_blockchain_convert_gmt_to_local_timestamp( $gmt_timestamp ) {
	$local = get_date_from_gmt( date( 'Y-m-d H:i:s', $gmt_timestamp ) );
	$local = strtotime( $local );

	return $local;
}
