<?php
/**
 * Ignition_Blockchain_Widget_Coinmarketcap_Single class
 *
 * @since 1.0.0
 */

/**
 * Class used to implement the Single Crypto Symbol widget.
 *
 * @since 1.0.0
 *
 * @see Ignition_Widget
 * @see WP_Widget
 */
class Ignition_Blockchain_Widget_Coinmarketcap_Single extends Ignition_Widget {

	protected $defaults = array(
		'title'       => '',
		'currency_id' => '1',
		'fiat'        => 'USD',
	);

	public function __construct() {
		$widget_ops  = array( 'description' => esc_html__( 'Displays information about a specific cryptocurrency.', 'ignition-blockchain' ) );
		$control_ops = array();

		parent::__construct( 'ignition-blockchain-coinmarketcap-single', esc_html__( 'Theme - Single Crypto Symbol', 'ignition-blockchain' ), $widget_ops, $control_ops );
	}

	public function widget( $args, $instance ) {
		$instance = wp_parse_args( (array) $instance, $this->defaults );

		$id            = isset( $args['id'] ) ? $args['id'] : '';
		$before_widget = $args['before_widget'];
		$after_widget  = $args['after_widget'];

		$title = apply_filters( 'widget_title', empty( $instance['title'] ) ? '' : $instance['title'], $instance, $this->id_base );

		$currency_id = $instance['currency_id'];
		$fiat        = $instance['fiat'];

		echo wp_kses( $before_widget, ignition_get_allowed_sidebar_wrappers() );

		if ( $title ) {
			echo wp_kses( $args['before_title'] . $title . $args['after_title'], ignition_get_allowed_sidebar_wrappers() );
		}

		$endpoint = 'cryptocurrency/quotes/latest';
		$params   = array(
			'id'      => $currency_id,
			'convert' => $fiat,
		);

		$response = ignition_blockchain_coinmarketcap_get_data( $endpoint, $params );

		if ( false === $response['error'] ) {
			$response_data = $response['data'];
			if ( ! empty( $response_data['data'] ) && is_array( $response_data['data'] ) ) {

				// In the pro API, the data are wrapped into an array, so, let's get the first element.
				$coin = reset( $response_data['data'] );

				$price = $coin['quote'][ $fiat ]['price'];
				?>
				<div
					class="widget-crypto widget-crypto-single loaded"
					data-currency="<?php echo esc_attr( $fiat ); ?>"
					data-symbol-id="<?php echo esc_attr( $currency_id ); ?>"
				>
					<div class="widget-crypto-placeholder">
						<div class="data-item-wrap">
							<div class="data-item-row">
								<div class="data-item">
									<span class="data-item-eyebrow js-symbol-name"><?php echo esc_html( $coin['name'] ); ?></span>
									<span class="data-item-value js-symbol"><?php echo esc_html( $coin['symbol'] ); ?></span>
								</div>

								<div class="data-item data-item-right">
									<span class="data-item-eyebrow"><?php esc_html_e( 'Price', 'ignition-blockchain' ); ?></span>
									<span class="data-item-value js-symbol-price"><?php echo esc_html( ignition_blockchain_coinmarketcap_format_currency_number( $price, $fiat ) ); ?></span>
								</div>
							</div>

							<div class="row">
								<?php $intervals = array( '1h', '24h', '7d' ); ?>
								<?php foreach ( $intervals as $interval ) : ?>
									<?php
										$change         = $coin['quote'][ $fiat ][ 'percent_change_' . strtolower( $interval ) ];
										$change_class   = '';
										$velocity_class = '';
										if ( floatval( $change ) < 0 ) {
											$velocity_class = 'symbol-desc';
											$change_class   = 'text-danger';
										} elseif ( floatval( $change ) > 0 ) {
											$velocity_class = 'symbol-asc';
											$change_class   = 'text-success';
										} else {
											$velocity_class = '';
											$change_class   = '';
										}
									?>
									<div class="col-4">
										<div class="data-item js-velocity" data-interval="<?php echo esc_attr( $interval ); ?>">
											<span class="data-item-eyebrow <?php echo esc_attr( $velocity_class ); ?>"><?php echo esc_html( $this->get_change_interval_short_text( $interval ) ); ?></span>
											<span class="data-item-value <?php echo esc_attr( $change_class ); ?>"><?php echo esc_html( sprintf( '%.2f%%', abs( $change ) ) ); ?></span>
										</div>
									</div>
								<?php endforeach; ?>
							</div>
						</div>

						<p class="data-last-updated">
							<?php esc_html_e( 'Last update:', 'ignition-blockchain' ); ?>
							<span class="data-last-updated-value">
								<?php
									$timestamp = strtotime( $coin['last_updated'] );
									echo esc_html( date_i18n( get_option( 'date_format' ) . ', ' . get_option( 'time_format' ), ignition_blockchain_convert_gmt_to_local_timestamp( $timestamp ) ) );
								?>
							</span>
						</p>
					</div>
				</div>
				<?php
			}
		} elseif ( true === $response['error'] && is_user_logged_in() && current_user_can( 'edit_theme_options' ) ) {
			foreach ( $response['errors'] as $error_msg ) {
				echo wp_kses_post( sprintf( '<p class="ignition-blockchain-coinmarketcap-error">%s</p>', $error_msg ) );
			}
		}

		echo wp_kses( $after_widget, ignition_get_allowed_sidebar_wrappers() );
	}

	public function update( $new_instance, $old_instance ) {
		$instance = $old_instance;

		$instance['title']       = sanitize_text_field( $new_instance['title'] );
		$instance['currency_id'] = absint( $new_instance['currency_id'] );
		$instance['fiat']        = ignition_blockchain_coinmarketcap_sanitize_fiat_currency( $new_instance['fiat'] );

		return $instance;
	}

	public function form( $instance ) {
		$instance = wp_parse_args( (array) $instance, $this->defaults );

		$this->field_input_text( 'title', array(
			'title' => __( 'Title:', 'ignition-blockchain' ),
			'value' => $instance['title'],
		) );

		$this->field_dropdown( 'currency_id', array(
			'title'        => __( 'Crypto Currency:', 'ignition-blockchain' ),
			'value'        => $instance['currency_id'],
			'choices'      => ignition_blockchain_coinmarketcap_get_cryptocurrency_choices(),
			'select_class' => 'widefat',
		) );

		$this->field_dropdown( 'fiat', array(
			'title'        => __( 'Show values expressed in:', 'ignition-blockchain' ),
			'value'        => $instance['fiat'],
			'choices'      => ignition_blockchain_coinmarketcap_get_fiat_currencies(),
			'select_class' => 'widefat',
		) );
	}

	protected function get_change_interval_short_text( $interval ) {
		$intervals = array(
			'1h'  => esc_html__( '1h', 'ignition-blockchain' ),
			'24h' => esc_html__( '24h', 'ignition-blockchain' ),
			'7d'  => esc_html__( '7d', 'ignition-blockchain' ),
		);

		if ( ! array_key_exists( $interval, $intervals ) ) {
			$interval = $this->defaults['interval'];
		}

		return $intervals[ $interval ];
	}

}
