<?php
/**
 * Ignition_Blockchain_Widget_Coinmarketcap_Ticker class
 *
 * @since 1.0.0
 */

/**
 * Class used to implement the Top Crypto Ticker widget.
 *
 * @since 1.0.0
 *
 * @see Ignition_Widget
 * @see WP_Widget
 */
class Ignition_Blockchain_Widget_Coinmarketcap_Ticker extends Ignition_Widget {

	protected $defaults = array(
		'title'    => '',
		'limit'    => 3,
		'start'    => 1,
		'fiat'     => 'USD',
		'interval' => '1h',
	);

	public function __construct() {
		$widget_ops  = array( 'description' => esc_html__( 'Displays a ticker of the top cryptocurrencies.', 'ignition-blockchain' ) );
		$control_ops = array();

		parent::__construct( 'ignition-blockchain-coinmarketcap-ticker', esc_html__( 'Theme - Top Crypto Ticker', 'ignition-blockchain' ), $widget_ops, $control_ops );
	}

	public function widget( $args, $instance ) {
		$instance = wp_parse_args( (array) $instance, $this->defaults );

		$id            = isset( $args['id'] ) ? $args['id'] : '';
		$before_widget = $args['before_widget'];
		$after_widget  = $args['after_widget'];

		$title = apply_filters( 'widget_title', empty( $instance['title'] ) ? '' : $instance['title'], $instance, $this->id_base );

		$start    = (int) $instance['start'];
		$start    = $instance['start'] < 1 ? 1 : $instance['start'];
		$limit    = $instance['limit'];
		$fiat     = $instance['fiat'];
		$interval = $instance['interval'];

		echo wp_kses( $before_widget, ignition_get_allowed_sidebar_wrappers() );

		if ( $title ) {
			echo wp_kses( $args['before_title'] . $title . $args['after_title'], ignition_get_allowed_sidebar_wrappers() );
		}

		$endpoint = 'cryptocurrency/listings/latest';
		$params   = array(
			'limit'   => $limit,
			'start'   => $start,
			'convert' => $fiat,
		);

		$response = ignition_blockchain_coinmarketcap_get_data( $endpoint, $params );

		$last_update = false;

		$p = $params;
		unset( $p['convert'] );
		$query_hash = ignition_blockchain_coinmarketcap_get_query_hash( 'ticker', $p );
		$noexp_name = ignition_blockchain_coinmarketcap_get_nonexpiring_transient_name( $query_hash );
		$noexp      = (array) get_transient( $noexp_name );
		if ( isset( $noexp['last_good_timestamp'] ) ) {
			$last_update = $noexp['last_good_timestamp'];
			$last_update = ignition_blockchain_convert_gmt_to_local_timestamp( $last_update );
		}
		unset( $p );

		if ( false === $response['error'] ) {
			$response_data = $response['data'];
			if ( ! empty( $response_data['data'] ) && is_array( $response_data['data'] ) ) {

				?>
				<div
					class="widget-crypto widget-crypto-multi loaded"
					data-currency="<?php echo esc_attr( $fiat ); ?>"
					data-limit="<?php echo esc_attr( $limit ); ?>"
					data-change-period="<?php echo esc_attr( $interval ); ?>"
					data-start="<?php echo esc_attr( $start ); ?>"
				>
					<div class="widget-crypto-placeholder">
						<div class="data-item-wrap data-item-list">
							<?php
								foreach ( $response_data['data'] as $coin ) {
									$price  = $coin['quote'][ $fiat ]['price'];
									$change = $coin['quote'][ $fiat ][ 'percent_change_' . strtolower( $interval ) ];

									$change_class = '';
									if ( floatval( $change ) < 0 ) {
										$change_class = 'text-danger symbol-desc';
									} elseif ( floatval( $change ) > 0 ) {
										$change_class = 'text-success symbol-asc';
									} else {
										$change_class = '';
									}
									?>
									<div class="data-item-row">
										<div class="data-item">
											<span class="data-item-eyebrow js-symbol-name"><?php echo esc_html( $coin['name'] ); ?></span>
											<span class="data-item-value js-symbol"><?php echo esc_html( $coin['symbol'] ); ?></span>
										</div>

										<div class="data-item data-item-right">
											<span class="data-item-eyebrow">
												<span class="js-symbol-change-interval"><?php echo esc_html( $this->get_change_interval_short_text( $interval ) ); ?></span>
												<span class="js-symbol-change <?php echo esc_attr( $change_class ); ?>"><?php echo esc_html( sprintf( '%.2f%%', abs( $change ) ) ); ?></span>
											</span>
											<span class="data-item-value js-symbol-price"><?php echo esc_html( ignition_blockchain_coinmarketcap_format_currency_number( $price, $fiat ) ); ?></span>
										</div>
									</div>
									<?php
								}
							?>
						</div>

						<?php if ( ! empty( $last_update ) ) : ?>
							<p class="data-last-updated">
								<?php esc_html_e( 'Last update:', 'ignition-blockchain' ); ?> <span class="data-last-updated-value"><?php echo esc_html( date_i18n( get_option( 'date_format' ) . ', ' . get_option( 'time_format' ), $last_update ) ); ?></span>
							</p>
						<?php endif; ?>

					</div>
				</div>
				<?php
			}
		} elseif ( true === $response['error'] && is_user_logged_in() && current_user_can( 'edit_theme_options' ) ) {
			foreach ( $response['errors'] as $error_msg ) {
				echo wp_kses_post( sprintf( '<p class="ignition-blockchain-coinmarketcap-error">%s</p>', $error_msg ) );
			}
		}

		echo wp_kses( $after_widget, ignition_get_allowed_sidebar_wrappers() );
	}

	public function update( $new_instance, $old_instance ) {
		$instance = $old_instance;

		$instance['title']    = sanitize_text_field( $new_instance['title'] );
		$instance['limit']    = absint( $new_instance['limit'] );
		$instance['start']    = absint( $new_instance['start'] );
		$instance['fiat']     = ignition_blockchain_coinmarketcap_sanitize_fiat_currency( $new_instance['fiat'] );
		$instance['interval'] = $this->sanitize_change_interval( $new_instance['interval'] );

		return $instance;
	}

	public function form( $instance ) {
		$instance = wp_parse_args( (array) $instance, $this->defaults );

		$this->field_input_text( 'title', array(
			'title' => __( 'Title:', 'ignition-blockchain' ),
			'value' => $instance['title'],
		) );

		$this->field_dropdown( 'limit', array(
			'title'        => __( 'Number of top cryptocurrencies:', 'ignition-blockchain' ),
			'value'        => $instance['limit'],
			'choices'      => $this->get_limit_options(),
			'select_class' => 'widefat',
		) );

		$this->field_input_int( 'start', array(
			'title'       => __( 'Start from the Nth top cryptocurrency:', 'ignition-blockchain' ),
			'value'       => $instance['start'],
			'input_attrs' => array(
				'min'  => 1,
				'max'  => 100,
				'step' => 1,
			),
		) );

		$this->field_dropdown( 'interval', array(
			'title'        => __( 'Show change percentage for the last:', 'ignition-blockchain' ),
			'value'        => $instance['interval'],
			'choices'      => $this->get_change_interval_options(),
			'select_class' => 'widefat',
		) );

		$this->field_dropdown( 'fiat', array(
			'title'        => __( 'Show values expressed in:', 'ignition-blockchain' ),
			'value'        => $instance['fiat'],
			'choices'      => ignition_blockchain_coinmarketcap_get_fiat_currencies(),
			'select_class' => 'widefat',
		) );
	}

	protected function get_limit_options() {
		return array(
			'0'   => esc_html__( 'All', 'ignition-blockchain' ),
			'3'   => '3',
			'5'   => '5',
			'10'  => '10',
			'25'  => '25',
			'50'  => '50',
			'100' => '100',
		);
	}

	protected function get_change_interval_options() {
		return array(
			'1h'  => esc_html__( '1 Hour', 'ignition-blockchain' ),
			'24h' => esc_html__( '1 Day', 'ignition-blockchain' ),
			'7d'  => esc_html__( '1 Week', 'ignition-blockchain' ),
		);
	}

	protected function get_change_interval_short_text( $interval ) {
		$intervals = array(
			'1h'  => esc_html__( '1h', 'ignition-blockchain' ),
			'24h' => esc_html__( '24h', 'ignition-blockchain' ),
			'7d'  => esc_html__( '7d', 'ignition-blockchain' ),
		);

		if ( ! array_key_exists( $interval, $intervals ) ) {
			$interval = $this->defaults['interval'];
		}

		return $intervals[ $interval ];
	}

	protected function sanitize_change_interval( $value ) {
		$choices = $this->get_change_interval_options();
		if ( array_key_exists( $value, $choices ) ) {
			return $value;
		}

		return $this->defaults['interval'];
	}
}
